// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.


package com.ibm.websphere.batch;

import java.util.Properties;



/**
 * <p>
 * The BatchDataStream (BDS) interface provides an abstraction over the data 
 * that is processed by a Batch Step. It provides the set of methods that are called 
 * by the Batch Execution Environment (BEE) in order to initialize the data stream,
 * retrieve cursor information from the BDS, and position the BDS to a specified cursor
 * during processing of a Batch Step.
 * 
 * </p>
 * 
 * This is the order in which methods are called on a BDS during normal execution of a batch step:
 * <ul>      
 *      <li> setProperties(Properties properties) </li>
 *      <li> initialize(String ilogicalname, String ijobstepid) </li>
 *      <li> open() </li>
 *      <li> positionAtInitialCheckpoint() </li>
 *      <li> externalizeCheckpointInformation() - this is called at every checkpoint </li>
 *      <li> intermediateCheckpoint() - this is called right after a checkpoint completes</li>
 *      <li> close() </li> 
 * 
 * </ul>
 * 
 * This is the order in which methods are called on a BDS when a batch step restarts processing a bds:
 * <ul>      
 *      <li> setProperties(Properties properties) </li>
 *      <li> initialize(String ilogicalname, String ijobstepid) </li>
 *      <li> open() </li>
 *      <li> internalizeCheckpointInformation(String chkptinfo) - informs the bds of the previously saved checkpoint information</li>
 *      <li> positionAtCurrentCheckpoint() - called to position the bds to the cursor indicated by the saved checkpoint in above method </li>
 *      <li> externalizeCheckpointInformation() - this is called at every checkpoint </li>
 *      <li> intermediateCheckpoint() - this is called right after a checkpoint completes</li>
 *      <li> close() </li> 
 * 
 * </ul>
 *
 * 
 * <p>
 * <b>Environment</b> The batch data stream is a java object that will be
 * called from both the Batch Execution Environment and the Batch Job Step EJB.
 * <ul>
 * <li><b>Transactional environment:</b> The caller of the BDS will be in a
 * global transaction when any of the methods on the interface are called.  
 * There is no guarantee that any two methods on the instance of the object 
 * will be called under the same global transactional unit of work. The global 
 * transactional unit of work is not owned by the BDS, the only transactional 
 * action that can be invoked by the BDS is to mark the global transaction as 
 * rollback only.
 * </li>
 * </ul>
 * </p>
 *
 * @ibm-api
 */



public interface BatchDataStream extends com.ibm.batch.api.BatchDataStream {
	
	
	/**
	 * <p>
	 * The externalizeCheckpointInformation method is called by the 
	 * Batch Execution Environment (BEE) during the checkpoint completion phase of processing. This 
	 * allows the BDS to record a set of information with the BEE, that 
	 * will be committed with the checkpoint's global transaction to the BEE database. The 
	 * information that is returned needs to contain enough data to 
	 * allow the BDS to reposition itself during a restart of the Batch Step
	 * Bean.
	 * </p>
	 * <p>Additional information can be stored in this string as well, 
	 * but the larger the string, the slower the Batch Execution Environment will be in storing 
	 * it in the RDB that it is maintaining.  
	 * @return - This is a string that is created to record the BDS's 
	 * position in the stream being processed. This String is opaque to 
	 * anyone but the BDS, so only the BDS can interpret its content.
	 */
	public String externalizeCheckpointInformation();

	/**
	 * <p>
	 * The internalizeCheckpointInformation method is called by the Batch Execution Environment 
	 * during the restart of a Batch Step Bean.  This allows the BDS to restart
	 * its internal state to the point it was at when the last successful 
	 * checkpoint was processed. It is at this point in the process that 
	 * the Batch Step Bean will begin processing.
	 * </p>
	 * @param chkptinfo - This is a string that was created by the 
	 * BDS that was returned to the Batch Execution Environment on the externalizeCheckpointInformation
	 * method. This String is opaque to anyone but the BDS, so only the 
	 * BDS can interpret its content.
	 */
	public void internalizeCheckpointInformation(String chkptinfo);
	
	/**
	 * <p>
	 * The initialize method is called by the Batch Execution Environment during the initialization of the job step.
	 * This allows the BDS to initialize the stream for use by the Batch Step Bean. The 
	 * method is passed the logical name of the BDS and the JobStepID. These two 
	 * pieces of information are used to create a BatchDataStreamConfig object that 
	 * has the configuration information necessary to establish the stream.  The 
	 * initialize method should configure the stream as defined by the config object
	 * and validate the input source or output sink (e.g. is the dataset available? 
	 * can I connect to the database being processed?). If the BDS is not valid,
	 * throw the BatchDataStreamConfigurationFailed exception.
	 * @param ilogicalname the logical name of the BDS, this name is used to 
	 * locate the resource in the job's xJCL.
	 * @param ijobstepid an identifier that represents the step within the batch 
	 * job that is being run. This id is used to locate the resource in the job's 
	 * xJCL.
	 * @throws BatchDataStreamConfigurationFailed is throw if the BDS is not 
	 * configured properly to access the BDS's source of data or its sink to 
	 * put data.
	 */

		public void initialize(String ilogicalname, String ijobstepid) throws BatchContainerDataStreamException;

		/**
		 * <p>
		 * The positionAtInitialCheckpoint is called by the Batch Execution Environment to provide a signal to the BDS that it should
		 * start processing the stream at the initial point as defined by the xJCL inputs.
		 *
		 */
		public void positionAtInitialCheckpoint() throws BatchContainerDataStreamException;

		/**
		 * <p>
		 * The positionAtCurrentCheckpoint method is called by Batch Execution Environment to provide a signal to the BDS that it 
		 * should start processing the stream at the point that was defined in the 
		 * internalizeCheckpointInformation method.
		 *
		 */
		public void positionAtCurrentCheckpoint() throws BatchContainerDataStreamException;

		/**
		 * <p>
		 * The open method is called by the Batch Execution Environment to indicate that the 
		 * BDS is about to be used and to prepare the BDS for operation. This 
		 * includes items such as opening files that are used by the BDS's
		 * implementation.
		 * </p>
		 *
		 */
		public void open() throws BatchContainerDataStreamException;

		/**
		 * The close method is called by the Batch Execution Environment to indicate to the BDS that the 
		 * user of the BDS is done working with the BDS. This 
		 * allows the BDS to free resources that it is holding.
		 */
		public void close() throws BatchContainerDataStreamException;
		
		/**
		 * The getName method returns the logical name of the BDS that was received in the initialize method.
		 * @return logical name of the BDS
		 */
		public String getName();

		
		/**
		 * <p>
		 * The setProperties method is called by the Batch Execution Environment to pass bds properties specified in xJCL to the
	     * bds as a java.util.Properties object.
	     * </p>	 
		 */        
	     public void setProperties(Properties properties);
		
	     /**
	      * <p>
	      * The getProperties method returns the bds properties specified in the xJCL and received in the setProperties 
	      * method.
	      * </p>
	      * @return bds properties defined in the xJCL
	      */
	     public Properties getProperties();
	     
	 	/**
	 	 * <p>
	 	 * The intermediateCheckpoint method is called by the Batch Execution Environment to indicate
	 	 * to the BDS that a checkpoint has just completed (i.e. a global transaction has been committed
	 	 * and a new global transaction has started). 
	 	 * </p>
	 	 */
	 	 public void intermediateCheckpoint();
	     
     

}
