// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2007
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office


package com.ibm.websphere.grid.spi;

import java.util.Properties;

/**
 * @ibm-spi
 *
 * <p>
 * WSGridNotificationsSPI is the base class for the WSGrid notification server
 * programming interface (SPI). Plugin to this SPI to intercept job related
 * notifications for jobs submitted to the XD Job Scheduler through the WSGrid 
 * job injector command line and API.  To add your own listener to receive 
 * WSGrid notifications, implement, install, and configure your own 
 * WSGridNoticationSPI subclass.
 * </p>
 *
 * <ol>
 * <li>
 * Implement by extending this class - e.g.
 * com.myCompany.xd.spi.MyWSGridNotificationSPI
 * </li>
 * <li>
 * Install by copying the jar file containing your implementation to 
 * your WebSphere classes directory. 
 * </li>
 * <li>
 * Configure by updating your xd.spi.properties file found in your
 * WebSphere profile's properties directory.  E.g.  
 * <pre>
 *    # WSGrid notification SPI
 *    # SPI name= WSGridNotificationSPI
 *    # WSGridNotificationSPI implementations are invoked in
 *    # numeric order given by the spi.WSGridNotificationSPI.{n}
 *    # property definitions - e.g. 
 *    # 
 *    # spi.WSGridNotificationSPI.1=com.abc.NotifySPI_A
 *    # spi.WSGridNotificationSPI.2=com.abc.NotifySPI_B
 *    # 
 *    # Would result in SPI implementations NotifySPI_A
 *    # and NotifySPI_B being invoked in that given order
 *    # each time a WSGrid notification is delivered.
 *         
 *    spi.WSGridNotificationSPI.1=com.myCompany.xd.spi.MyWSGridNotificationSPI
 * </pre>   
 * </li>
 * </ol>
 *
 * <p>
 * Note your WSGridNotificationSPI subclass implementation is invoked inside the application 
 * server process in which the XD Job Scheduler runs.
 * </p>
 *
 */

public abstract class WSGridNotificationSPI extends SPI {
	
    /**
     * <p>Identifies the job started notification type.</p>
     */
    public static final String JOB_STATE_STARTED= "job.state.started"; 
    /**
     * <p>Identifies the step started notification type.</p>
     */
    public static final String STEP_STATE_STARTED= "step.state.started";
    /**
     * <p>Identifies the step ended notification type.</p>
     */
    public static final String STEP_STATE_ENDED= "step.state.ended";
    /**
     * <p>Identifies the job ended notification type.</p>
     */
    public static final String JOB_STATE_ENDED= "job.state.ended";

	/**
	 * <p>
	 * Callback method invoked by job scheduler job lifecycle events. This method
	 * must be implemented by the WSGridNotificationSPI subclass implementor.
	 * </p>
	 * @param jobProps contains the properties specified in the WSGrid properties file(s) for this job.
	 *
	 * @param xdJobID specifies the job id of the job corresponding to this notification.
	 *
	 * @param notification specifies the lifecycle event that occurred for the specified job.  
	 * The event type is defined by the following constants: 
	 * <ol>
	 * <li>JOB_STATE_STARTED</li>
	 * <li>STEP_STATE_STARTED</li>
	 * <li>STEP_STATE_ENDED</li>
	 * <li>JOB_STATE_ENDED</li>
	 * </ol>
	 * @param rc specifies the step or job return code. It is defined only for notification types:
	 * <ol>
	 * <li>STEP_STATE_ENDED</li>
	 * <li>JOB_STATE_ENDED</li>
	 * </ol>
	 */
	public abstract void notificationReceived(Properties jobProps, String xdJobID, String notification, int rc);

}

