package com.ibm.wsspi.batch.parallel.status;

import com.ibm.websphere.grid.spi.SPI;
import java.io.Externalizable;

/**
 * SubJobCollector is the abstract base class for the ParallelJobManager SubJobCollector System Programming Interface (SPI).
 * Use the SubJobCollector to collect data about a specific subjob. The SubJobCollector is invoked immediately before a 
 * checkpoint is taken by a subjob instance.
 * <p>  
 * Implement the SubJobCollector SPI, by extending this class and implementing the collect method.
 * <p>
 * Configure this SPI with the following declaration in the xd.spi.properties file:
 * <p>
 * <pre>
 * spi.parallel.status.SubJobCollector=&lt;fully-qualified implementation class name&gt;
 * </pre>
 * <p>
 * <b>Notes:</b> 
 * <ol>
 * <li>The xd.spi.properties file is located in:</li>
 * <p>
 * <pre>
 * &lt;WAS install root&gt;/properties.  E.g.
 * 
 *       /WebSphere/AppServer/properties
 * </pre>
 * <li>The jar file containing the implementation of this SPI should be stored in:</li>
 * <p>
 * <pre>
 * &lt;WAS install root&gt;/lib/classes.  E.g.
 * 
 *       /WebSphere/AppServer/lib/classes
 * </pre> 
 * <p>
 * You are responsible for creating the classes directory if it does not already exist. 
 * @ibm-spi
 */
public abstract class SubJobCollector extends SPI {

	/**
	 * Collects data for the specified subjob.  This data is then transmitted to the ParallelJobManager
	 * for delivery to the SubJobAnalyzer SPI.  This method is called immediately before a checkpoint
	 * is taken.   
	 * @param parallelJobName specifies name of the current parallel job. 
	 * This name is the same each time you submit the same parallel job.  
	 * @param logicalTXID specifies a unique identifier for the current parallel job instance.  
	 * This identifier defines a logical unit of work in which all subjobs for the current parallel job execute. 
	 * @param subJobID specifies the unique identifier of a subjob belonging to the current parallel job.
	 * @return Externalizable containing data to be delivered by ParallelJobManager to SubJobAnalyzer SPI.
	 * If return value is null, delivery is cancelled and SubJobAnalyzer will not be called.  
	 * @see com.ibm.wsspi.batch.parallel.status.SubJobAnalyzer SubJobAnalyzer
	 */
	public abstract Externalizable collect(String parallelJobName, String logicalTXID, String subJobID);
	
	
	public String getName() 
	{
		return "SubJobCollector";
	}
}
